# This file is part of Seppo - The Form Analysator
# Copyright (C) 2002  Matti Lattu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package form;

sub new {
  my $class = shift;
  
  my $self = {};
  %$self = ();
  
  # Define package-wide vars
  $items = 0;
  $scalemin = 0;
  $scalemax = 0;
  @item = ();
  @item_name = ();
  @index = ();
  @index_name = ();
  @index_formula = ();
  @index_formula_value_cache = ();
  @response = ();

  bless $self;
  }
  
sub setConf {
  my $class = shift;
  
  my %par = @_;

  my $key;
  my $cmd;
  my $n;
  
  foreach $key (keys %par) {
    $cmd = lc($key);
    $cmd =~ s/[^a-z_]//g;
    
    $n = $key;
    $n =~ s/\D//g;
    
    # print "cmd: $cmd, n: $n, par: ".$par{$key}."\n";
    
    if ($cmd eq 'items') { $items = $par{$key}; }
    if ($cmd eq 'scalemin') { $scalemin = $par{$key}; }
    if ($cmd eq 'scalemax') { $scalemax = $par{$key}; }
    
    if ($cmd eq 'item_name') { $item_name[$n] = $par{$key}; }
    if ($cmd eq 'index_name') { $index_name[$n] = $par{$key}; }
    if ($cmd eq 'index_formula') { $index_formula[$n] = $par{$key}; }
    
    if ($cmd eq 'response') { $response[$n] = $par{$key}; }
    
    }
  
  return 1;  
  }

sub getItems {
  my $class = shift;
  
  return $items;
  }
  
sub getItem {
  my $class = shift;
  
  my $which_index = shift;
  $which_index =~ s/\D//g;
  
  if ($which_index ne '') {
    return $item[$which_index];
    }
  else {
    return @item;
    }
  }

sub getItemList {
  my $class = shift;
  
  my $item_list = shift,
  
  $item_list =~ s/[^\s\dqir]//gi;
  
  my @ret = ();
  my $key;
  my $no;
  
  foreach (split(/\s/,$item_list)) {
    $key = lc($_);
    $key =~ s/[^qir]//g;
    
    $no = $_;
    $no =~ s/[^\d]//gi;
    
    if ($key eq 'q') {
      push (@ret,&getItem($class,$no));
      }
    if ($key eq 'i') {
      push(@ret,&getIndexFormulaValue($class,$no));
      }
    if ($key eq 'r') {
      push(@ret,&getResponse($class,$no));
      }
    }
    
  return (@ret);
  }
  
sub getItemName {
  my $class = shift;
  
  my $which_index = shift;
  $which_index =~ s/\D//g;
  
  if ($which_index ne '') {
    return $item_name[$which_index];
    }
  else {
    return @item_name;
    }
  }
  
sub getIndexName {
  my $class = shift;
  
  my $which_index = shift;
  $which_index =~ s/\D//g;
  
  if ($which_index ne '') {
    return $index_name[$which_index];
    }
  else {
    return @index_name;
    }
  }

sub getIndexFormula {
  my $class = shift;
  
  my $which_index = shift;
  $which_index =~ s/\D//g;
  
  if ($which_index ne '') {
    return $index_formula[$which_index];
    }
  else {
    return @index_formula;
    }
  }
  
sub getIndexFormulaValue {
  my $class = shift;
  
  my $which_index = shift;
  $which_index =~ s/\D//g;
  
  if ($which_index ne '') {
    if ($index_formula_value_cache[$which_index] ne '') {
      return $index_formula_value_cache[$which_index];
      }
      
    my $formula = $index_formula[$which_index];
    
    $formula =~ s/q(\d+)/\$item[$1]/g;
    
    my $value = eval "($formula)";
    if ($@) {
      &error("Error in your formula (".$index_formula[$which_index]."): $@");
      return undef;
      }
    
    $index_formula_value_cache[$which_index] = $value;
    return $value;
    }
  else {
    my @value_arr = ();
    for (my $n=0; $n <= scalar(@index_formula); $n++ ) {
      if ($index_formula[$n] ne '') {
        $value_arr[$n] = &getIndexFormulaValue($class,$n);
	}
      }
    return @value_arr;
    }
  
  }

sub getResponse {
  my $class = shift;
  
  my $which_index = shift;
  $which_index =~ s/\D//g;
  
  if ($which_index ne '') {
    my @pairs = split(/\x02/,$response[$which_index]);
    
    my $condition;
    my $response;

    my $ret_response = '';
    
    my @index_formula_value = &getIndexFormulaValue($class);
    
    foreach my $pair (@pairs) {
      ($condition,$response) = split(/\x01/,$pair,2);

      my $orig_condition = $condition;
            
      $condition =~ s/q(\d+)/\$item[$1]/g;
      $condition =~ s/i(\d+)/\$index_formula_value[$1]/g;
      
      my $true = eval "($condition)";
      
      if ($@) {
        &error("Error in your response condition (".$orig_condition."): $@");
	next;
        }
      
      if ($true) {
        $ret_response = $response;
        last;
	}
      }

    return $ret_response;
    }
  else {  
    &error("getResponse: array mode is not implemented");
    }
  }
  
sub getTagsAll {

  # Returns a hash containing all relevant data
  
  my $class = shift;
  
  my %ret = ();
  
  my @item_names = &getItemName($class);
  my @items = &getItem($class);
  
  foreach my $n (0..(scalar(@item_names)-1)) {
    $ret{"item_name_$n"} = $item_names[$n];
    }

  foreach my $n (0..(scalar(@items)-1)) {
    $ret{"item_$n"} = $items[$n];
    }

  my @index_formulas = &getIndexFormula($class);
  for (my $n=0; $n <= scalar(@index_formulas); $n++) {
    if ($index_formulas[$n] ne '') {
      $ret{"index_name_$n"} = &getIndexName($class,$n);
      $ret{"index_formula_$n"} = $index_formulas[$n];
      $ret{"index_$n"} = &getIndexFormulaValue($class,$n);
      }
    }
  
  for (my $n=0; $n <= scalar(@response); $n++) {
    if ($response[$n] ne '') {
      $ret{"response_$n"} = &getResponse($class,$n);
      }
    }
    
  return %ret;
  }
    
sub getTagsAllSafe {
  
  # Like getAllTags, but makes safety filtering (HTML) for tag values
  
  my $class = shift;
  
  my %ret = &getTagsAll($class);
  
  foreach my $key (keys %ret) {
    $ret{$key} =~ s/[<\x8b]/&lt;/g;
    $ret{$key} =~ s/[>\x9b]/&gt;/g;
    }
  
  return %ret;
  }
  

  
sub setItem {
  my $class = shift;
  
  my %par = @_;

  my $key;
  my $cmd;
  my $n;
  
  foreach $key (keys %par) {
    $cmd = lc($key);
    $cmd =~ s/[^a-z_]//g;
    
    $n = $key;
    $n =~ s/\D//g;
    
    if ($cmd eq 'q') { $item[$n] = $par{$key}; }
    }
  
  return 1;
  }
  
sub setItemArray {
  my $class = shift;
  
  my @new_item = @_;
  
  if (scalar(@new_item) != $items) {
    print STDERR "setItemArray: length of new item array does not match pre-set item length\n";
    return 0;
    }
    
  @item = @new_item;
  
  return 1;
  }

sub error {
  print "form.pm error: ".$_[0]."\n";
  return;
  }
    
1;
